@@ -3,14 +3,13 @@ package main
 import (
 	"context"
 	"fmt"
-	"io"
+	"log"
 	"os"
 	"reflect"
 	"strings"
 	"time"
 
 	"go.mongodb.org/mongo-driver/mongo"
-	"gopkg.in/yaml.v2"
 
 	"github.com/percona/percona-backup-mongodb/pbm/config"
 	"github.com/percona/percona-backup-mongodb/pbm/connect"
@@ -18,15 +17,14 @@ import (
 	"github.com/percona/percona-backup-mongodb/sdk"
 )
 
-const resyncWaitDuration = 30 * time.Second
-
 type configOpts struct {
-	rsync bool
-	wait  bool
-	list  bool
-	file  string
-	set   map[string]string
-	key   string
+	rsync    bool
+	wait     bool
+	waitTime time.Duration
+	list     bool
+	file     string
+	set      map[string]string
+	key      string
 }
 
 type confKV struct {
@@ -49,7 +47,18 @@ func (c confVals) String() string {
 	return s
 }
 
-func runConfig(ctx context.Context, conn connect.Client, pbm sdk.Client, c *configOpts) (fmt.Stringer, error) {
+func runConfig(
+	ctx context.Context,
+	conn connect.Client,
+	pbm *sdk.Client,
+	c *configOpts,
+) (fmt.Stringer, error) {
+	if len(c.set) != 0 || c.rsync || c.file != "" {
+		if err := checkForAnotherOperation(ctx, pbm); err != nil {
+			return nil, err
+		}
+	}
+
 	switch {
 	case len(c.set) > 0:
 		var o confVals
@@ -75,6 +84,9 @@ func runConfig(ctx context.Context, conn connect.Client, pbm sdk.Client, c *conf
 	case len(c.key) > 0:
 		k, err := config.GetConfigVar(ctx, conn, c.key)
 		if err != nil {
+			if errors.Is(err, config.ErrUnsetConfigPath) {
+				return confKV{c.key, ""}, nil // unset config path
+			}
 			return nil, errors.Wrap(err, "unable to get config key")
 		}
 		return confKV{c.key, fmt.Sprint(k)}, nil
@@ -88,35 +100,33 @@ func runConfig(ctx context.Context, conn connect.Client, pbm sdk.Client, c *conf
 			return outMsg{"Storage resync started"}, nil
 		}
 
-		ctx, cancel := context.WithTimeout(ctx, resyncWaitDuration)
-		defer cancel()
+		if c.waitTime > time.Second {
+			var cancel context.CancelFunc
+			ctx, cancel = context.WithTimeout(ctx, c.waitTime)
+			defer cancel()
+		}
 
 		err = sdk.WaitForResync(ctx, pbm, cid)
 		if err != nil {
 			if errors.Is(err, context.DeadlineExceeded) {
-				err = errors.New("timeout")
+				err = errWaitTimeout
 			}
+
 			return nil, errors.Wrapf(err, "waiting for resync [opid %q]", cid)
 		}
 
 		return outMsg{"Storage resync finished"}, nil
 	case len(c.file) > 0:
-		var buf []byte
 		var err error
+		var newCfg *config.Config
 
 		if c.file == "-" {
-			buf, err = io.ReadAll(os.Stdin)
+			newCfg, err = config.Parse(os.Stdin)
 		} else {
-			buf, err = os.ReadFile(c.file)
+			newCfg, err = readConfigFromFile(c.file)
 		}
 		if err != nil {
-			return nil, errors.Wrap(err, "unable to read config file")
-		}
-
-		var newCfg config.Config
-		err = yaml.UnmarshalStrict(buf, &newCfg)
-		if err != nil {
-			return nil, errors.Wrap(err, "unable to  unmarshal config file")
+			return nil, errors.Wrap(err, "unable to get new config")
 		}
 
 		oldCfg, err := pbm.GetConfig(ctx)
@@ -127,7 +137,7 @@ func runConfig(ctx context.Context, conn connect.Client, pbm sdk.Client, c *conf
 			oldCfg = &config.Config{}
 		}
 
-		if err := config.SetConfig(ctx, conn, &newCfg); err != nil {
+		if err := config.SetConfig(ctx, conn, newCfg); err != nil {
 			return nil, errors.Wrap(err, "unable to set config: write to db")
 		}
 
@@ -143,3 +153,17 @@ func runConfig(ctx context.Context, conn connect.Client, pbm sdk.Client, c *conf
 
 	return pbm.GetConfig(ctx)
 }
+
+func readConfigFromFile(filename string) (*config.Config, error) {
+	file, err := os.Open(filename)
+	if err != nil {
+		return nil, errors.Wrapf(err, "open %q", err)
+	}
+	defer func() {
+		if err := file.Close(); err != nil {
+			log.Printf("close: %v", err)
+		}
+	}()
+
+	return config.Parse(file)
+}
