@@ -25,6 +25,7 @@ import (
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
 	"github.com/percona/percona-backup-mongodb/pbm/topo"
 	"github.com/percona/percona-backup-mongodb/pbm/util"
+	"github.com/percona/percona-backup-mongodb/sdk"
 )
 
 type restoreOpts struct {
@@ -39,6 +40,8 @@ type restoreOpts struct {
 	rsMap         string
 	conf          string
 	ts            string
+
+	numParallelColls int32
 }
 
 type restoreRet struct {
@@ -97,7 +100,17 @@ func (r externRestoreRet) String() string {
 		r.Name, r.Name)
 }
 
-func runRestore(ctx context.Context, conn connect.Client, o *restoreOpts, outf outFormat) (fmt.Stringer, error) {
+func runRestore(
+	ctx context.Context,
+	conn connect.Client,
+	pbm *sdk.Client,
+	o *restoreOpts,
+	outf outFormat,
+) (fmt.Stringer, error) {
+	numParallelColls, err := parseCLINumParallelCollsOption(o.numParallelColls)
+	if err != nil {
+		return nil, errors.Wrap(err, "parse --num-parallel-collections option")
+	}
 	nss, err := parseCLINSOption(o.ns)
 	if err != nil {
 		return nil, errors.Wrap(err, "parse --ns option")
@@ -115,13 +128,17 @@ func runRestore(ctx context.Context, conn connect.Client, o *restoreOpts, outf o
 		return nil, errors.New("either a backup name or point in time should be set, non both together!")
 	}
 
+	if err := checkForAnotherOperation(ctx, pbm); err != nil {
+		return nil, err
+	}
+
 	clusterTime, err := topo.GetClusterTime(ctx, conn)
 	if err != nil {
 		return nil, errors.Wrap(err, "read cluster time")
 	}
 	tdiff := time.Now().Unix() - int64(clusterTime.T)
 
-	m, err := doRestore(ctx, conn, o, nss, rsMap, outf)
+	m, err := doRestore(ctx, conn, o, numParallelColls, nss, rsMap, outf)
 	if err != nil {
 		return nil, err
 	}
@@ -310,6 +327,7 @@ func doRestore(
 	ctx context.Context,
 	conn connect.Client,
 	o *restoreOpts,
+	numParallelColls *int32,
 	nss []string,
 	rsMapping map[string]string,
 	outf outFormat,
@@ -318,22 +336,19 @@ func doRestore(
 	if err != nil {
 		return nil, err
 	}
-	err = checkConcurrentOp(ctx, conn)
-	if err != nil {
-		return nil, err
-	}
 
 	name := time.Now().UTC().Format(time.RFC3339Nano)
 
 	cmd := ctrl.Cmd{
 		Cmd: ctrl.CmdRestore,
 		Restore: &ctrl.RestoreCmd{
-			Name:          name,
-			BackupName:    bcp,
-			Namespaces:    nss,
-			UsersAndRoles: o.usersAndRoles,
-			RSMap:         rsMapping,
-			External:      o.extern,
+			Name:             name,
+			BackupName:       bcp,
+			NumParallelColls: numParallelColls,
+			Namespaces:       nss,
+			UsersAndRoles:    o.usersAndRoles,
+			RSMap:            rsMapping,
+			External:         o.extern,
 		},
 	}
 	if o.pitr != "" {
@@ -532,6 +547,7 @@ type describeRestoreResult struct {
 	Namespaces         []string         `json:"namespaces,omitempty" yaml:"namespaces,omitempty"`
 	StartTS            *int64           `json:"start_ts,omitempty" yaml:"-"`
 	StartTime          *string          `json:"start,omitempty" yaml:"start,omitempty"`
+	FinishTime         *string          `json:"finish,omitempty" yaml:"finish,omitempty"`
 	PITR               *int64           `json:"ts_to_restore,omitempty" yaml:"-"`
 	PITRTime           *string          `json:"time_to_restore,omitempty" yaml:"time_to_restore,omitempty"`
 	LastTransitionTS   int64            `json:"last_transition_ts" yaml:"-"`
@@ -618,18 +634,16 @@ func describeRestore(ctx context.Context, conn connect.Client, o descrRestoreOpt
 	res.OPID = meta.OPID
 	res.LastTransitionTS = meta.LastTransitionTS
 	res.LastTransitionTime = time.Unix(res.LastTransitionTS, 0).UTC().Format(time.RFC3339)
+	res.StartTime = util.Ref(time.Unix(meta.StartTS, 0).UTC().Format(time.RFC3339))
+	if meta.Status == defs.StatusDone {
+		res.FinishTime = util.Ref(time.Unix(meta.LastTransitionTS, 0).UTC().Format(time.RFC3339))
+	}
 	if meta.Status == defs.StatusError {
 		res.Error = &meta.Error
 	}
-	if meta.StartPITR != 0 {
-		res.StartTS = &meta.StartPITR
-		s := time.Unix(meta.StartPITR, 0).UTC().Format(time.RFC3339)
-		res.StartTime = &s
-	}
 	if meta.PITR != 0 {
 		res.PITR = &meta.PITR
-		s := time.Unix(meta.PITR, 0).UTC().Format(time.RFC3339)
-		res.PITRTime = &s
+		res.PITRTime = util.Ref(time.Unix(meta.PITR, 0).UTC().Format(time.RFC3339))
 	}
 
 	for _, rs := range meta.Replsets {
