@@ -15,7 +15,6 @@ import (
 
 	"github.com/percona/percona-backup-mongodb/pbm/archive"
 	"github.com/percona/percona-backup-mongodb/pbm/compress"
-	"github.com/percona/percona-backup-mongodb/pbm/config"
 	"github.com/percona/percona-backup-mongodb/pbm/connect"
 	"github.com/percona/percona-backup-mongodb/pbm/ctrl"
 	"github.com/percona/percona-backup-mongodb/pbm/defs"
@@ -37,6 +36,12 @@ func (b *Backup) doLogical(
 	stg storage.Storage,
 	l log.LogEvent,
 ) error {
+	if b.brief.ConfigSvr {
+		if err := b.checkForTimeseries(ctx, bcp.Namespaces); err != nil {
+			return errors.Wrap(err, "check for timeseries")
+		}
+	}
+
 	var db, coll string
 	if util.IsSelective(bcp.Namespaces) {
 		// for selective backup, configsvr does not hold any data.
@@ -67,7 +72,8 @@ func (b *Backup) doLogical(
 	}
 
 	if inf.IsLeader() {
-		err := b.reconcileStatus(ctx, bcp.Name, opid.String(), defs.StatusRunning, ref(b.timeouts.StartingStatus()))
+		err := b.reconcileStatus(ctx,
+			bcp.Name, opid.String(), defs.StatusRunning, util.Ref(b.timeouts.StartingStatus()))
 		if err != nil {
 			if errors.Is(err, errConvergeTimeOut) {
 				return errors.Wrap(err, "couldn't get response from all shards")
@@ -75,16 +81,18 @@ func (b *Backup) doLogical(
 			return errors.Wrap(err, "check cluster for backup started")
 		}
 
+		// TODO(improve): do setClusterFirstWrite between
+		// all replsets status are StatusRunning and setting the global status
 		err = b.setClusterFirstWrite(ctx, bcp.Name)
 		if err != nil {
 			return errors.Wrap(err, "set cluster first write ts")
 		}
-	}
-
-	// Waiting for cluster's StatusRunning to move further.
-	err = b.waitForStatus(ctx, bcp.Name, defs.StatusRunning, nil)
-	if err != nil {
-		return errors.Wrap(err, "waiting for running")
+	} else {
+		// Waiting for cluster's StatusRunning to move further.
+		err = b.waitForStatus(ctx, bcp.Name, defs.StatusRunning, nil)
+		if err != nil {
+			return errors.Wrap(err, "waiting for running")
+		}
 	}
 
 	stopOplogSlicer := startOplogSlicer(ctx,
@@ -129,17 +137,24 @@ func (b *Backup) doLogical(
 	if len(nssSize) == 0 {
 		dump = snapshot.DummyBackup{}
 	} else {
-		dump, err = snapshot.NewBackup(b.brief.URI, b.dumpConns, db, coll)
+		numParallelColls := b.numParallelColls
+		if bcp.NumParallelColls != nil {
+			if *bcp.NumParallelColls > 0 {
+				numParallelColls = int(*bcp.NumParallelColls)
+			} else {
+				l.Warning("invalid value of NumParallelCollections (%v). fallback to %v",
+					numParallelColls, b.numParallelColls)
+			}
+		}
+
+		l.Debug("dumping up to %d collections in parallel", numParallelColls)
+
+		dump, err = snapshot.NewBackup(b.brief.URI, numParallelColls, db, coll)
 		if err != nil {
 			return errors.Wrap(err, "init mongodump options")
 		}
 	}
 
-	cfg, err := config.GetConfig(ctx, b.leadConn)
-	if err != nil {
-		return errors.Wrap(err, "get config")
-	}
-
 	nsFilter := archive.DefaultNSFilter
 	docFilter := archive.DefaultDocFilter
 	if inf.IsConfigSrv() && util.IsSelective(bcp.Namespaces) {
@@ -155,7 +170,7 @@ func (b *Backup) doLogical(
 	snapshotSize, err := snapshot.UploadDump(ctx,
 		dump,
 		func(ns, ext string, r io.Reader) error {
-			stg, err := util.StorageFromConfig(cfg.Storage, l)
+			stg, err := util.StorageFromConfig(&b.config.Storage, l)
 			if err != nil {
 				return errors.Wrap(err, "get storage")
 			}
@@ -184,11 +199,11 @@ func (b *Backup) doLogical(
 		if err != nil {
 			return errors.Wrap(err, "check cluster for dump done")
 		}
-	}
-
-	err = b.waitForStatus(ctx, bcp.Name, defs.StatusDumpDone, nil)
-	if err != nil {
-		return errors.Wrap(err, "waiting for dump done")
+	} else {
+		err = b.waitForStatus(ctx, bcp.Name, defs.StatusDumpDone, nil)
+		if err != nil {
+			return errors.Wrap(err, "waiting for dump done")
+		}
 	}
 
 	lastSavedTS, oplogSize, err := stopOplogSlicer()
@@ -446,3 +461,32 @@ func getNamespacesSize(ctx context.Context, m *mongo.Client, db, coll string) (m
 	err = eg.Wait()
 	return rv, err
 }
+
+func (b *Backup) checkForTimeseries(ctx context.Context, nss []string) error {
+	if !b.brief.Version.IsShardedTimeseriesSupported() || !b.brief.Sharded {
+		return nil
+	}
+
+	tss, err := topo.ListShardedTimeseries(ctx, b.leadConn)
+	if err != nil {
+		return errors.Wrap(err, "list sharded timeseries")
+	}
+
+	if util.IsSelective(nss) {
+		selected := util.MakeSelectedPred(nss)
+		origTSs := tss
+		tss = make([]string, 0, len(tss))
+		for _, ts := range origTSs {
+			if selected(ts) {
+				tss = append(tss, ts)
+			}
+		}
+	}
+
+	if len(tss) != 0 {
+		return errors.Errorf("cannot backup following sharded timeseries: %s",
+			strings.Join(tss, ", "))
+	}
+
+	return nil
+}
