@@ -14,21 +14,25 @@ import (
 	"fmt"
 	"io"
 	"reflect"
+	"slices"
 	"strings"
 	"sync/atomic"
 
 	"github.com/mongodb/mongo-tools/common/bsonutil"
 	"github.com/mongodb/mongo-tools/common/db"
+	"github.com/mongodb/mongo-tools/common/dumprestore"
 	"github.com/mongodb/mongo-tools/common/idx"
 	"github.com/mongodb/mongo-tools/common/txn"
 	"github.com/mongodb/mongo-tools/mongorestore/ns"
-	"github.com/pkg/errors"
 	"go.mongodb.org/mongo-driver/bson"
 	"go.mongodb.org/mongo-driver/bson/primitive"
 	"go.mongodb.org/mongo-driver/mongo"
 
-	"github.com/percona/percona-backup-mongodb/pbm"
+	"github.com/percona/percona-backup-mongodb/pbm/defs"
+	"github.com/percona/percona-backup-mongodb/pbm/errors"
+	"github.com/percona/percona-backup-mongodb/pbm/restore/phys"
 	"github.com/percona/percona-backup-mongodb/pbm/snapshot"
+	"github.com/percona/percona-backup-mongodb/pbm/version"
 )
 
 type Record = db.Oplog
@@ -41,8 +45,8 @@ func DefaultOpFilter(*Record) bool { return true }
 
 var excludeFromOplog = []string{
 	"config.rangeDeletions",
-	pbm.DB + "." + pbm.TmpUsersCollection,
-	pbm.DB + "." + pbm.TmpRolesCollection,
+	defs.DB + "." + defs.TmpUsersCollection,
+	defs.DB + "." + defs.TmpRolesCollection,
 }
 
 var knownCommands = map[string]struct{}{
@@ -74,8 +78,6 @@ var selectedNSSupportedCommands = map[string]struct{}{
 	"dropIndex":        {},
 	"dropIndexes":      {},
 	"collMod":          {},
-	"startIndexBuild":  {},
-	"abortIndexBuild":  {},
 	"commitIndexBuild": {},
 }
 
@@ -84,6 +86,7 @@ var dontPreserveUUID = []string{
 	"admin.system.roles",
 	"admin.system.keys",
 	"*.system.buckets.*", // timeseries
+	"*.system.views",     // timeseries
 }
 
 // OplogRestore is the oplog applyer
@@ -104,7 +107,7 @@ type OplogRestore struct {
 	// the queue of last N committed transactions
 	txnCommit *cqueue
 
-	txn        chan pbm.RestoreTxn
+	txn        chan phys.RestoreTxn
 	txnSyncErr chan error
 	// The `T` part of the last applied op's Timestamp.
 	// Keeping just `T` allows atomic use as we only care
@@ -126,10 +129,10 @@ const saveLastDistTxns = 100
 func NewOplogRestore(
 	dst *mongo.Client,
 	ic *idx.IndexCatalog,
-	sv *pbm.MongoVersion,
+	sv *version.MongoVersion,
 	unsafe,
 	preserveUUID bool,
-	ctxn chan pbm.RestoreTxn,
+	ctxn chan phys.RestoreTxn,
 	txnErr chan error,
 ) (*OplogRestore, error) {
 	m, err := ns.NewMatcher(append(snapshot.ExcludeFromRestore, excludeFromOplog...))
@@ -254,6 +257,32 @@ func (o *OplogRestore) SetIncludeNS(nss []string) {
 	o.includeNS = dbs
 }
 
+func isOpAllowed(oe *Record) bool {
+	coll, ok := strings.CutPrefix(oe.Namespace, "config.")
+	if !ok {
+		return true // OK: not a "config" database. allow any ops
+	}
+
+	if slices.Contains(dumprestore.ConfigCollectionsToKeep, coll) {
+		return true // OK: create/update/delete a doc
+	}
+
+	if coll != "$cmd" || len(oe.Object) == 0 {
+		return false // other collection is not allowed
+	}
+
+	op := oe.Object[0].Key
+	if op == "applyOps" {
+		return true // internal ops of applyOps are checked one by one later
+	}
+	if _, ok := selectedNSSupportedCommands[op]; ok {
+		s, _ := oe.Object[0].Value.(string)
+		return slices.Contains(dumprestore.ConfigCollectionsToKeep, s)
+	}
+
+	return false
+}
+
 func (o *OplogRestore) isOpSelected(oe *Record) bool {
 	if o.includeNS == nil || o.includeNS[""] != nil {
 		return true
@@ -269,11 +298,13 @@ func (o *OplogRestore) isOpSelected(oe *Record) bool {
 		return false
 	}
 
-	for _, el := range oe.Object {
-		if _, ok := selectedNSSupportedCommands[el.Key]; ok {
-			s, _ := el.Value.(string)
-			return colls[s]
-		}
+	cmd := oe.Object[0].Key
+	if cmd == "applyOps" {
+		return true // internal ops of applyOps are checked one by one later
+	}
+	if _, ok := selectedNSSupportedCommands[cmd]; ok {
+		s, _ := oe.Object[0].Value.(string)
+		return colls[s]
 	}
 
 	return false
@@ -289,16 +320,16 @@ func (o *OplogRestore) handleOp(oe db.Oplog) error {
 		return nil
 	}
 
-	if o.excludeNS.Has(oe.Namespace) {
-		return nil
-	}
-
 	// skip no-ops
 	if oe.Operation == "n" {
 		return nil
 	}
 
-	if !o.isOpSelected(&oe) {
+	if o.excludeNS.Has(oe.Namespace) {
+		return nil
+	}
+
+	if !isOpAllowed(&oe) || !o.isOpSelected(&oe) {
 		return nil
 	}
 
@@ -315,7 +346,7 @@ func (o *OplogRestore) handleOp(oe db.Oplog) error {
 	if o.cnamespase != oe.Namespace {
 		o.preserveUUID = o.preserveUUIDopt
 
-		// if this is a create operation, the namesape would be
+		// if this is a create operation, the namespace would be
 		// inside the object to create
 		if oe.Operation == "c" {
 			if len(oe.Object) == 0 {
@@ -448,10 +479,10 @@ func (o *OplogRestore) handleTxnOp(meta txn.Meta, op db.Oplog) error {
 			}
 		}
 
-		o.txnCommit.push(pbm.RestoreTxn{
+		o.txnCommit.push(phys.RestoreTxn{
 			ID:    txnID,
 			Ctime: cts,
-			State: pbm.TxnCommit,
+			State: phys.TxnCommit,
 		})
 	}
 
@@ -565,7 +596,7 @@ func bsonDocToOplog(doc bson.D) (*db.Oplog, error) {
 func (o *OplogRestore) applyTxn(id string) error {
 	t, ok := o.txnData[id]
 	if !ok {
-		return errors.Errorf("unknown transaction id %s", id)
+		return nil
 	}
 
 	for _, op := range t.applyOps {
@@ -580,7 +611,7 @@ func (o *OplogRestore) applyTxn(id string) error {
 }
 
 //nolint:nonamedreturns
-func (o *OplogRestore) TxnLeftovers() (uncommitted map[string]Txn, lastCommits []pbm.RestoreTxn) {
+func (o *OplogRestore) TxnLeftovers() (uncommitted map[string]Txn, lastCommits []phys.RestoreTxn) {
 	return o.txnData, o.txnCommit.s
 }
 
@@ -621,11 +652,16 @@ func (o *OplogRestore) handleNonTxnOp(op db.Oplog) error {
 		return nil
 	}
 
+	if !isOpAllowed(&op) || !o.isOpSelected(&op) {
+		return nil
+	}
+
 	op, err := o.filterUUIDs(op)
 	if err != nil {
 		return errors.Wrap(err, "filtering UUIDs from oplog")
 	}
 
+	dbName, collName, _ := strings.Cut(op.Namespace, ".")
 	if op.Operation == "c" {
 		if len(op.Object) == 0 {
 			return errors.Errorf("empty object value for op: %v", op)
@@ -636,9 +672,6 @@ func (o *OplogRestore) handleNonTxnOp(op db.Oplog) error {
 			return errors.Errorf("unknown oplog command name %v: %v", cmdName, op)
 		}
 
-		ns := strings.Split(op.Namespace, ".")
-		dbName := ns[0]
-
 		switch cmdName {
 		case "commitIndexBuild":
 			// commitIndexBuild was introduced in 4.4, one "commitIndexBuild" command can contain several
@@ -755,7 +788,20 @@ func (o *OplogRestore) handleNonTxnOp(op db.Oplog) error {
 			op2 := op
 			op2.Object = bson.D{{"drop", collName}}
 			if err := o.handleNonTxnOp(op2); err != nil {
-				return errors.WithMessage(err, "oplog: drop collection before create")
+				return errors.Wrap(err, "oplog: drop collection before create")
+			}
+		}
+	} else if op.Operation == "i" && collName == "system.views" {
+		// PBM-921: ensure the collection exists before "creating" views or timeseries
+		err := o.dst.Database(dbName).CreateCollection(context.TODO(), "system.views")
+		if err != nil {
+			// MongoDB 5.0 and 6.0 returns NamespaceExists error.
+			// MongoDB 7.0 and 8.0 does not return error.
+			// https://github.com/mongodb/mongo/blob/v6.0/src/mongo/base/error_codes.yml#L84
+			const NamespaceExists = 48
+			var cmdError mongo.CommandError
+			if !errors.As(err, &cmdError) || cmdError.Code != NamespaceExists {
+				return errors.Wrapf(err, "ensure %s.system.views collection", dbName)
 			}
 		}
 	}
@@ -764,7 +810,7 @@ func (o *OplogRestore) handleNonTxnOp(op db.Oplog) error {
 	if err != nil {
 		// https://jira.percona.com/browse/PBM-818
 		if o.unsafe && op.Namespace == "config.chunks" {
-			if se, ok := err.(mongo.ServerError); ok && se.HasErrorCode(11000) { //nolint:errorlint
+			if mongo.IsDuplicateKeyError(err) {
 				return nil
 			}
 		}
@@ -777,15 +823,15 @@ func (o *OplogRestore) handleNonTxnOp(op db.Oplog) error {
 }
 
 type cqueue struct {
-	s []pbm.RestoreTxn
+	s []phys.RestoreTxn
 	c int
 }
 
 func newCQueue(capacity int) *cqueue {
-	return &cqueue{s: make([]pbm.RestoreTxn, 0, capacity), c: capacity}
+	return &cqueue{s: make([]phys.RestoreTxn, 0, capacity), c: capacity}
 }
 
-func (c *cqueue) push(v pbm.RestoreTxn) {
+func (c *cqueue) push(v phys.RestoreTxn) {
 	if len(c.s) == c.c {
 		c.s = c.s[1:]
 	}
@@ -793,7 +839,7 @@ func (c *cqueue) push(v pbm.RestoreTxn) {
 	c.s = append(c.s, v)
 }
 
-func (c *cqueue) last() *pbm.RestoreTxn {
+func (c *cqueue) last() *phys.RestoreTxn {
 	if len(c.s) == 0 {
 		return nil
 	}
