@@ -6,6 +6,7 @@ import (
 	"fmt"
 	"io"
 	"path"
+	"slices"
 	"strings"
 	"time"
 
@@ -43,7 +44,11 @@ type Restore struct {
 	brief    topo.NodeBrief
 	stopHB   chan struct{}
 	nodeInfo *topo.NodeInfo
-	stg      storage.Storage
+	cfg      *config.Config
+	bcpStg   storage.Storage
+	oplogStg storage.Storage
+
+	numParallelColls int
 	// Shards to participate in restore. Num of shards in bcp could
 	// be less than in the cluster and this is ok. Only these shards
 	// would be expected to run restore (distributed transactions sync,
@@ -64,11 +69,24 @@ type Restore struct {
 	indexCatalog *idx.IndexCatalog
 }
 
+type oplogRange struct {
+	chunks []oplog.OplogChunk
+
+	storage storage.Storage
+}
+
 // PBM restore from temp collections (pbmRUsers/pbmRRoles)should be used
 type restoreUsersAndRolesOption bool
 
 // New creates a new restore object
-func New(leadConn connect.Client, nodeConn *mongo.Client, brief topo.NodeBrief, rsMap map[string]string) *Restore {
+func New(
+	leadConn connect.Client,
+	nodeConn *mongo.Client,
+	brief topo.NodeBrief,
+	cfg *config.Config,
+	rsMap map[string]string,
+	numParallelColls int,
+) *Restore {
 	if rsMap == nil {
 		rsMap = make(map[string]string)
 	}
@@ -79,6 +97,10 @@ func New(leadConn connect.Client, nodeConn *mongo.Client, brief topo.NodeBrief,
 		brief:    brief,
 		rsMap:    rsMap,
 
+		cfg: cfg,
+
+		numParallelColls: numParallelColls,
+
 		indexCatalog: idx.NewIndexCatalog(),
 	}
 }
@@ -141,19 +163,26 @@ func shouldRestoreUsersAndRoles(nssBackup, nssRestore []string, usingUsersAndRol
 // Snapshot do the snapshot's (mongo dump) restore
 //
 //nolint:nonamedreturns
-func (r *Restore) Snapshot(ctx context.Context, cmd *ctrl.RestoreCmd, opid ctrl.OPID, l log.LogEvent) (err error) {
-	defer func() { r.exit(log.Copy(context.Background(), ctx), err) }()
+func (r *Restore) Snapshot(
+	ctx context.Context,
+	cmd *ctrl.RestoreCmd,
+	opid ctrl.OPID,
+	bcp *backup.BackupMeta,
+) (err error) {
+	l := log.LogEventFromContext(ctx)
 
-	bcp, err := SnapshotMeta(ctx, r.leadConn, cmd.BackupName, r.stg)
-	if err != nil {
-		return err
-	}
+	defer func() { r.exit(log.Copy(context.Background(), ctx), err) }()
 
 	err = r.init(ctx, cmd.Name, opid, l)
 	if err != nil {
 		return err
 	}
 
+	r.bcpStg, err = util.StorageFromConfig(&bcp.Store.StorageConf, r.log)
+	if err != nil {
+		return errors.Wrap(err, "get backup storage")
+	}
+
 	nss := resolveNamespace(bcp.Namespaces, cmd.Namespaces, cmd.UsersAndRoles)
 	usersAndRolesOpt := shouldRestoreUsersAndRoles(bcp.Namespaces, cmd.Namespaces, cmd.UsersAndRoles)
 
@@ -162,7 +191,7 @@ func (r *Restore) Snapshot(ctx context.Context, cmd *ctrl.RestoreCmd, opid ctrl.
 		return errors.Wrap(err, "set backup name")
 	}
 
-	err = r.checkSnapshot(ctx, bcp)
+	err = r.checkSnapshot(ctx, bcp, nss)
 	if err != nil {
 		return err
 	}
@@ -196,13 +225,15 @@ func (r *Restore) Snapshot(ctx context.Context, cmd *ctrl.RestoreCmd, opid ctrl.
 		return err
 	}
 
+	oplogRanges := []oplogRange{
+		{chunks: chunks, storage: r.bcpStg},
+	}
 	oplogOption := &applyOplogOption{end: &bcp.LastWriteTS, nss: nss}
 	if r.nodeInfo.IsConfigSrv() && util.IsSelective(nss) {
 		oplogOption.nss = []string{"config.databases"}
 		oplogOption.filter = newConfigsvrOpFilter(nss)
 	}
-
-	err = r.applyOplog(ctx, chunks, oplogOption)
+	err = r.applyOplog(ctx, oplogRanges, oplogOption)
 	if err != nil {
 		return err
 	}
@@ -245,7 +276,14 @@ func newConfigsvrOpFilter(nss []string) oplog.OpFilter {
 // PITR do the Point-in-Time Recovery
 //
 //nolint:nonamedreturns
-func (r *Restore) PITR(ctx context.Context, cmd *ctrl.RestoreCmd, opid ctrl.OPID, l log.LogEvent) (err error) {
+func (r *Restore) PITR(
+	ctx context.Context,
+	cmd *ctrl.RestoreCmd,
+	opid ctrl.OPID,
+	bcp *backup.BackupMeta,
+) (err error) {
+	l := log.LogEventFromContext(ctx)
+
 	defer func() { r.exit(log.Copy(context.Background(), ctx), err) }()
 
 	err = r.init(ctx, cmd.Name, opid, l)
@@ -253,15 +291,20 @@ func (r *Restore) PITR(ctx context.Context, cmd *ctrl.RestoreCmd, opid ctrl.OPID
 		return err
 	}
 
-	bcp, err := SnapshotMeta(ctx, r.leadConn, cmd.BackupName, r.stg)
-	if err != nil {
-		return errors.Wrap(err, "get base backup")
-	}
 	if bcp.LastWriteTS.Compare(cmd.OplogTS) >= 0 {
 		return errors.New("snapshot's last write is later than the target time. " +
 			"Try to set an earlier snapshot. Or leave the snapshot empty so PBM will choose one.")
 	}
 
+	r.bcpStg, err = util.StorageFromConfig(&bcp.Store.StorageConf, r.log)
+	if err != nil {
+		return errors.Wrap(err, "get backup storage")
+	}
+	r.oplogStg, err = util.GetStorage(ctx, r.leadConn, log.LogEventFromContext(ctx))
+	if err != nil {
+		return errors.Wrap(err, "get oplog storage")
+	}
+
 	nss := resolveNamespace(bcp.Namespaces, cmd.Namespaces, cmd.UsersAndRoles)
 	usersAndRolesOpt := shouldRestoreUsersAndRoles(bcp.Namespaces, cmd.Namespaces, cmd.UsersAndRoles)
 
@@ -277,7 +320,7 @@ func (r *Restore) PITR(ctx context.Context, cmd *ctrl.RestoreCmd, opid ctrl.OPID
 		return errors.Wrap(err, "set backup name")
 	}
 
-	err = r.checkSnapshot(ctx, bcp)
+	err = r.checkSnapshot(ctx, bcp, nss)
 	if err != nil {
 		return err
 	}
@@ -292,7 +335,7 @@ func (r *Restore) PITR(ctx context.Context, cmd *ctrl.RestoreCmd, opid ctrl.OPID
 		bcpShards[i] = bcp.Replsets[i].Name
 	}
 
-	if !Contains(bcpShards, util.MakeReverseRSMapFunc(r.rsMap)(r.nodeInfo.SetName)) {
+	if !slices.Contains(bcpShards, util.MakeReverseRSMapFunc(r.rsMap)(r.nodeInfo.SetName)) {
 		return r.Done(ctx) // skip. no backup for current rs
 	}
 
@@ -325,13 +368,16 @@ func (r *Restore) PITR(ctx context.Context, cmd *ctrl.RestoreCmd, opid ctrl.OPID
 		return err
 	}
 
+	oplogRanges := []oplogRange{
+		{chunks: bcpChunks, storage: r.bcpStg},
+		{chunks: chunks, storage: r.oplogStg},
+	}
 	oplogOption := applyOplogOption{end: &cmd.OplogTS, nss: nss}
 	if r.nodeInfo.IsConfigSrv() && util.IsSelective(nss) {
 		oplogOption.nss = []string{"config.databases"}
 		oplogOption.filter = newConfigsvrOpFilter(nss)
 	}
-
-	err = r.applyOplog(ctx, append(bcpChunks, chunks...), &oplogOption)
+	err = r.applyOplog(ctx, oplogRanges, &oplogOption)
 	if err != nil {
 		return err
 	}
@@ -382,10 +428,15 @@ func (r *Restore) ReplayOplog(ctx context.Context, cmd *ctrl.ReplayCmd, opid ctr
 		return errors.Wrap(err, "topology")
 	}
 
-	if !Contains(oplogShards, util.MakeReverseRSMapFunc(r.rsMap)(r.nodeInfo.SetName)) {
+	if !slices.Contains(oplogShards, util.MakeReverseRSMapFunc(r.rsMap)(r.nodeInfo.SetName)) {
 		return r.Done(ctx) // skip. no oplog for current rs
 	}
 
+	r.oplogStg, err = util.GetStorage(ctx, r.leadConn, log.LogEventFromContext(ctx))
+	if err != nil {
+		return errors.Wrapf(err, "get oplog storage")
+	}
+
 	opChunks, err := r.chunks(ctx, cmd.Start, cmd.End)
 	if err != nil {
 		return err
@@ -396,12 +447,16 @@ func (r *Restore) ReplayOplog(ctx context.Context, cmd *ctrl.ReplayCmd, opid ctr
 		return err
 	}
 
+	oplogRanges := []oplogRange{
+		{chunks: opChunks, storage: r.oplogStg},
+	}
 	oplogOption := applyOplogOption{
 		start:  &cmd.Start,
 		end:    &cmd.End,
 		unsafe: true,
 	}
-	if err = r.applyOplog(ctx, opChunks, &oplogOption); err != nil {
+	err = r.applyOplog(ctx, oplogRanges, &oplogOption)
+	if err != nil {
 		return err
 	}
 
@@ -480,11 +535,6 @@ func (r *Restore) init(ctx context.Context, name string, opid ctrl.OPID, l log.L
 		return errors.Wrap(err, "add shard's metadata")
 	}
 
-	r.stg, err = util.GetStorage(ctx, r.leadConn, r.log)
-	if err != nil {
-		return errors.Wrap(err, "get backup storage")
-	}
-
 	return nil
 }
 
@@ -517,21 +567,35 @@ func (r *Restore) checkTopologyForOplog(currShards []topo.Shard, oplogShards []s
 // is contiguous - there are no gaps), checks for respective files on storage and returns
 // chunks list if all checks passed
 func (r *Restore) chunks(ctx context.Context, from, to primitive.Timestamp) ([]oplog.OplogChunk, error) {
-	return chunks(ctx, r.leadConn, r.stg, from, to, r.nodeInfo.SetName, r.rsMap)
+	return chunks(ctx, r.leadConn, r.oplogStg, from, to, r.nodeInfo.SetName, r.rsMap)
 }
 
-func SnapshotMeta(
+// LookupBackupMeta fetches backup metadata.
+//
+// It tries to find the metadata in database. If there is no such metadata in
+// database, it tries to fetch from the main storage.
+func LookupBackupMeta(
 	ctx context.Context,
 	conn connect.Client,
 	backupName string,
-	stg storage.Storage,
 ) (*backup.BackupMeta, error) {
 	bcp, err := backup.NewDBManager(conn).GetBackupByName(ctx, backupName)
-	if errors.Is(err, errors.ErrNotFound) {
-		bcp, err = GetMetaFromStore(stg, backupName)
+	if err == nil {
+		return bcp, nil
 	}
+	if !errors.Is(err, errors.ErrNotFound) {
+		return nil, errors.Wrap(err, "get backup metadata from db")
+	}
+
+	var stg storage.Storage
+	stg, err = util.GetStorage(ctx, conn, log.LogEventFromContext(ctx))
 	if err != nil {
-		return nil, errors.Wrap(err, "get backup metadata")
+		return nil, errors.Wrap(err, "get storage")
+	}
+
+	bcp, err = GetMetaFromStore(stg, backupName)
+	if err != nil {
+		return nil, errors.Wrap(err, "get backup metadata from storage")
 	}
 
 	return bcp, nil
@@ -596,11 +660,11 @@ func (r *Restore) snapshotObjects(bcp *backup.BackupMeta) (string, []oplog.Oplog
 		return "", nil, ErrNoDataForShard
 	}
 
-	if _, err := r.stg.FileStat(rsMeta.DumpName); err != nil {
+	if _, err := r.bcpStg.FileStat(rsMeta.DumpName); err != nil {
 		return "", nil, errors.Wrapf(err, "failed to ensure snapshot file %s", rsMeta.DumpName)
 	}
 	if version.IsLegacyBackupOplog(bcp.PBMVersion) {
-		if _, err := r.stg.FileStat(rsMeta.OplogName); err != nil {
+		if _, err := r.bcpStg.FileStat(rsMeta.OplogName); err != nil {
 			return "", nil, errors.Errorf("failed to ensure oplog file %s: %v", rsMeta.OplogName, err)
 		}
 
@@ -614,7 +678,7 @@ func (r *Restore) snapshotObjects(bcp *backup.BackupMeta) (string, []oplog.Oplog
 		return rsMeta.DumpName, chunks, nil
 	}
 
-	files, err := r.stg.List(rsMeta.OplogName, "")
+	files, err := r.bcpStg.List(rsMeta.OplogName, "")
 	if err != nil {
 		return "", nil, errors.Wrap(err, "failed to list oplog files")
 	}
@@ -637,7 +701,7 @@ func (r *Restore) snapshotObjects(bcp *backup.BackupMeta) (string, []oplog.Oplog
 	return rsMeta.DumpName, chunks, nil
 }
 
-func (r *Restore) checkSnapshot(ctx context.Context, bcp *backup.BackupMeta) error {
+func (r *Restore) checkSnapshot(ctx context.Context, bcp *backup.BackupMeta, nss []string) error {
 	if bcp.Status != defs.StatusDone {
 		return errors.Errorf("backup wasn't successful: status: %s, error: %s",
 			bcp.Status, bcp.Error())
@@ -671,6 +735,16 @@ func (r *Restore) checkSnapshot(ctx context.Context, bcp *backup.BackupMeta) err
 				bcp.MongoVersion, ver.VersionString)
 			return nil
 		}
+
+		if r.brief.Sharded && ver.IsConfigShardSupported() && util.IsSelective(nss) {
+			hasConfigShard, err := topo.HasConfigShard(ctx, r.leadConn)
+			if err != nil {
+				return errors.Wrap(err, "check for Config Shard")
+			}
+			if hasConfigShard {
+				return errors.New("selective restore is not supported with Config Shard")
+			}
+		}
 	}
 
 	return nil
@@ -689,10 +763,9 @@ func (r *Restore) RunSnapshot(
 	usersAndRolesOpt restoreUsersAndRolesOption,
 ) error {
 	var rdr io.ReadCloser
-
 	var err error
 	if version.IsLegacyArchive(bcp.PBMVersion) {
-		sr, err := r.stg.SourceReader(dump)
+		sr, err := r.bcpStg.SourceReader(dump)
 		if err != nil {
 			return errors.Wrapf(err, "get object %s for the storage", dump)
 		}
@@ -724,19 +797,11 @@ func (r *Restore) RunSnapshot(
 			// so we'll continue with selective restore
 		}
 
-		var cfg *config.Config
-		// get pbm.Config for creating a storage.Storage later.
-		// while r.stg is already created storage for the restore,
-		// it triggers data race warnings during concurrent file downloading/reading.
-		// for that, it's better to create a new storage for each file
-		cfg, err = config.GetConfig(ctx, r.leadConn)
-		if err != nil {
-			return errors.Wrap(err, "get config")
-		}
+		r.log.Debug("restoring up to %d collections in parallel", r.numParallelColls)
 
 		rdr, err = snapshot.DownloadDump(
 			func(ns string) (io.ReadCloser, error) {
-				stg, err := util.StorageFromConfig(cfg.Storage, r.log)
+				stg, err := util.StorageFromConfig(&bcp.Store.StorageConf, r.log)
 				if err != nil {
 					return nil, errors.Wrap(err, "get storage")
 				}
@@ -765,7 +830,8 @@ func (r *Restore) RunSnapshot(
 				return rdr, nil
 			},
 			bcp.Compression,
-			util.MakeSelectedPred(nss))
+			util.MakeSelectedPred(nss),
+			r.numParallelColls)
 	}
 	if err != nil {
 		return err
@@ -773,7 +839,7 @@ func (r *Restore) RunSnapshot(
 	defer rdr.Close()
 
 	// Restore snapshot (mongorestore)
-	err = r.snapshot(ctx, rdr)
+	err = r.snapshot(rdr)
 	if err != nil {
 		return errors.Wrap(err, "mongorestore")
 	}
@@ -1086,15 +1152,22 @@ func (r *Restore) getcommittedTxn(ctx context.Context) (map[string]primitive.Tim
 	return txn, nil
 }
 
-func (r *Restore) applyOplog(ctx context.Context, chunks []oplog.OplogChunk, options *applyOplogOption) error {
+func (r *Restore) applyOplog(ctx context.Context, ranges []oplogRange, options *applyOplogOption) error {
 	mgoV, err := version.GetMongoVersion(ctx, r.nodeConn)
 	if err != nil || len(mgoV.Version) < 1 {
 		return errors.Wrap(err, "define mongo version")
 	}
 	stat := phys.RestoreShardStat{}
-	partial, err := applyOplog(ctx, r.nodeConn, chunks, options, r.nodeInfo.IsSharded(),
-		r.indexCatalog, r.setcommittedTxn, r.getcommittedTxn, &stat.Txn,
-		&mgoV, r.stg, r.log)
+	partial, err := applyOplog(ctx,
+		r.nodeConn,
+		ranges,
+		options,
+		r.nodeInfo.IsSharded(),
+		r.indexCatalog,
+		r.setcommittedTxn,
+		r.getcommittedTxn,
+		&stat.Txn,
+		&mgoV)
 	if err != nil {
 		return errors.Wrap(err, "reply oplog")
 	}
@@ -1119,13 +1192,8 @@ func (r *Restore) applyOplog(ctx context.Context, chunks []oplog.OplogChunk, opt
 	return nil
 }
 
-func (r *Restore) snapshot(ctx context.Context, input io.Reader) error {
-	cfg, err := config.GetConfig(ctx, r.leadConn)
-	if err != nil {
-		return errors.Wrap(err, "unable to get PBM config settings")
-	}
-
-	rf, err := snapshot.NewRestore(r.brief.URI, cfg)
+func (r *Restore) snapshot(input io.Reader) error {
+	rf, err := snapshot.NewRestore(r.brief.URI, r.cfg, r.numParallelColls)
 	if err != nil {
 		return err
 	}
