@@ -34,7 +34,7 @@ func (r *Restore) configsvrRestore(
 	mapRS util.RSMapFunc,
 ) error {
 	mapS := util.MakeRSMapFunc(r.sMap)
-	available, err := fetchAvailability(bcp, r.stg)
+	available, err := fetchAvailability(bcp, r.bcpStg)
 	if err != nil {
 		return err
 	}
@@ -123,7 +123,7 @@ func (r *Restore) configsvrRestoreDatabases(
 	mapRS, mapS util.RSMapFunc,
 ) error {
 	filepath := path.Join(bcp.Name, mapRS(r.brief.SetName), "config.databases"+bcp.Compression.Suffix())
-	rdr, err := r.stg.SourceReader(filepath)
+	rdr, err := r.bcpStg.SourceReader(filepath)
 	if err != nil {
 		return err
 	}
@@ -206,7 +206,7 @@ func (r *Restore) configsvrRestoreCollections(
 	}
 
 	filepath := path.Join(bcp.Name, mapRS(r.brief.SetName), "config.collections"+bcp.Compression.Suffix())
-	rdr, err := r.stg.SourceReader(filepath)
+	rdr, err := r.bcpStg.SourceReader(filepath)
 	if err != nil {
 		return nil, err
 	}
@@ -270,7 +270,7 @@ func (r *Restore) configsvrRestoreChunks(
 	mapS util.RSMapFunc,
 ) error {
 	filepath := path.Join(bcp.Name, mapRS(r.brief.SetName), "config.chunks"+bcp.Compression.Suffix())
-	rdr, err := r.stg.SourceReader(filepath)
+	rdr, err := r.bcpStg.SourceReader(filepath)
 	if err != nil {
 		return err
 	}
