@@ -19,6 +19,7 @@ import (
 	"github.com/percona/percona-backup-mongodb/pbm/lock"
 	"github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/oplog"
+	"github.com/percona/percona-backup-mongodb/pbm/prio"
 	"github.com/percona/percona-backup-mongodb/pbm/restore"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
 	"github.com/percona/percona-backup-mongodb/pbm/topo"
@@ -51,7 +52,7 @@ func NewSlicer(
 		leadClient: cn,
 		node:       node,
 		rs:         rs,
-		span:       int64(defs.PITRdefaultSpan),
+		span:       int64(defs.DefaultPITRInterval),
 		storage:    to,
 		oplog:      oplog.NewOplogBackup(node),
 		cfg:        cfg,
@@ -117,13 +118,6 @@ func (s *Slicer) Catchup(ctx context.Context) error {
 		return nil
 	}
 
-	if lastBackup.Type != defs.LogicalBackup {
-		// the backup does not contain complete oplog to copy from
-		// NOTE: the chunk' last op can be later than backup' first write ts
-		s.lastTS = lastChunk.EndTS
-		return nil
-	}
-
 	if !lastChunk.EndTS.Before(rs.LastWriteTS) {
 		// no need to copy oplog from backup
 		s.lastTS = lastChunk.EndTS
@@ -138,19 +132,31 @@ func (s *Slicer) Catchup(ctx context.Context) error {
 			return errors.Wrap(err, "get config")
 		}
 
-		err = s.upload(ctx, lastChunk.EndTS, rs.FirstWriteTS, cfg.PITR.Compression, cfg.PITR.CompressionLevel)
+		err = s.upload(ctx,
+			lastChunk.EndTS,
+			rs.FirstWriteTS,
+			cfg.PITR.Compression,
+			cfg.PITR.CompressionLevel)
 		if err != nil {
-			return err
-		}
+			var rangeErr oplog.InsuffRangeError
+			if !errors.As(err, &rangeErr) {
+				return err
+			}
 
-		s.l.Info("uploaded chunk %s - %s", formatts(lastChunk.EndTS), formatts(rs.FirstWriteTS))
-		s.lastTS = rs.FirstWriteTS
+			s.l.Warning("skip chunk %s - %s: oplog has insufficient range",
+				formatts(lastChunk.EndTS), formatts(rs.FirstWriteTS))
+		} else {
+			s.l.Info("uploaded chunk %s - %s", formatts(lastChunk.EndTS), formatts(rs.FirstWriteTS))
+			s.lastTS = rs.FirstWriteTS
+		}
 	}
 
-	err = s.copyReplsetOplog(ctx, rs)
-	if err != nil {
-		s.l.Error("copy oplog from %q backup: %v", lastBackup.Name, err)
-		return nil
+	if lastBackup.Type == defs.LogicalBackup {
+		err = s.copyReplsetOplog(ctx, rs)
+		if err != nil {
+			s.l.Error("copy oplog from %q backup: %v", lastBackup.Name, err)
+			return nil
+		}
 	}
 	s.lastTS = rs.LastWriteTS
 
@@ -273,11 +279,13 @@ const LogStartMsg = "start_ok"
 // Stream streaming (saving) chunks of the oplog to the given storage
 func (s *Slicer) Stream(
 	ctx context.Context,
+	startingNode *topo.NodeInfo,
 	stopC <-chan struct{},
 	backupSig <-chan ctrl.OPID,
 	compression compress.CompressionType,
 	level *int,
 	timeouts *config.BackupTimeouts,
+	monitorPrio bool,
 ) error {
 	if s.lastTS.T == 0 {
 		return errors.New("no starting point defined")
@@ -288,11 +296,6 @@ func (s *Slicer) Stream(
 	tk := time.NewTicker(cspan)
 	defer tk.Stop()
 
-	nodeInfo, err := topo.GetNodeInfoExt(ctx, s.node)
-	if err != nil {
-		return errors.Wrap(err, "get NodeInfo data")
-	}
-
 	// early check for the log sufficiency to display error
 	// before the timer clicks (not to wait minutes to report)
 	ok, err := s.oplog.IsSufficient(s.lastTS)
@@ -388,9 +391,15 @@ func (s *Slicer) Stream(
 		if ld.Type != ctrl.CmdPITR {
 			return errors.Errorf("another operation is running: %v", ld)
 		}
-		if ld.Node != nodeInfo.Me {
+		if ld.Node != startingNode.Me {
 			return OpMovedError{ld.Node}
 		}
+		if monitorPrio &&
+			prio.CalcPriorityForNode(startingNode) > prio.CalcPriorityForNode(ninf) {
+			return errors.Errorf("node priority has changed %.1f->%.1f",
+				prio.CalcPriorityForNode(startingNode),
+				prio.CalcPriorityForNode(ninf))
+		}
 		if sliceTo.IsZero() {
 			majority, err := topo.IsWriteMajorityRequested(ctx, s.node, s.leadClient.MongoOptions().WriteConcern)
 			if err != nil {
@@ -402,21 +411,6 @@ func (s *Slicer) Stream(
 			}
 		}
 
-		// if this is the last slice, epoch probably already changed (e.g. due to config changes) and that's ok
-		if !lastSlice {
-			cep, err := config.GetEpoch(ctx, s.leadClient)
-			if err != nil {
-				return errors.Wrap(err, "get epoch")
-			}
-
-			if !s.cfg.Epoch.Equal(cep.TS()) {
-				return errors.Errorf(
-					"epoch mismatch. Got sleep in %v, woke up in %v. Too old for that stuff.",
-					s.cfg.Epoch, cep.TS(),
-				)
-			}
-		}
-
 		err = s.upload(ctx, s.lastTS, sliceTo, compression, level)
 		if err != nil {
 			return err
