@@ -98,6 +98,7 @@ func DownloadDump(
 	download DownloadFunc,
 	compression compress.CompressionType,
 	match archive.NSFilterFn,
+	numParallelColls int,
 ) (io.ReadCloser, error) {
 	pr, pw := io.Pipe()
 
@@ -120,7 +121,7 @@ func DownloadDump(
 			return r, errors.Wrapf(err, "create decompressor: %q", ns)
 		}
 
-		err := archive.Compose(pw, match, newReader)
+		err := archive.Compose(pw, newReader, match, numParallelColls)
 		pw.CloseWithError(errors.Wrap(err, "compose"))
 	}()
 
